#include "lpc23xx.h"
#include "ethernetdrv.h"

void write_PHY (int PhyReg, int Value) {
  unsigned int tout;

  MAC_MADR = (1<<8) | PhyReg;
  MAC_MWTD = Value;

  /* Wait utill operation completed */
  tout = 0;
  for (tout = 0; tout < 0x00050000; tout++) {
    if ((MAC_MIND & 0x00000001) == 0) {
      break;
    }
  }
}

unsigned short read_PHY (unsigned char PhyReg) {
  unsigned int tout;

  MAC_MADR = (1<<8) | PhyReg;
  MAC_MCMD = 0x00000001;

  /* Wait until operation completed */
  tout = 0;
  for (tout = 0; tout < 0x00050000; tout++) {
    if ((MAC_MIND & 0x00000001) == 0) {
      break;
    }
  }
  MAC_MCMD = 0;
  return (MAC_MRDD);
}

int ethernet_isup(void){
	return (read_PHY(1) & 4);
}

int get_phyid(void){
	return (read_PHY(2));
}

int get_phyrev(void){
	return (read_PHY(3) & 0xF);
}

int get_phymodel(void){
	return ((read_PHY(3)>>4) & 0x3F);
}

int MAC_init(){
	int tout,i;
	PCONP |= (1<<30);
	PINSEL2 |= 0x50150105;
	PINSEL3 |= 0x5;
	
	//Reset MAC
	MAC_MAC1 = 0x00000100 | 0x000002009 | 0x00000400 | 0x00000800 | 0x00004000 | 0x00008000;
	MAC_COMMAND = (1<<3) | (1<<4) | (1<<5);
	
	
	for (tout = 0; tout<10000; tout++);
	MAC_MAC1 = 0x00000002;
	MAC_MAC2 = 0x00000010 | 0x00000020;
	MAC_MAXF = 1522;
	MAC_CLRT = 0x0000370F;
	MAC_IPGR = 0x00000012;
	
	/* Enable Reduced MII interface. */
	MAC_COMMAND = 0x00000200 | 0x00000040;
	
	write_PHY (0, 0x8000); //send reset command
	while(read_PHY(0) & 0x8000); //wait until reset is complete
	
	MAC_MCFG = (0x06 << 2) | 0x00008000;	
	for ( i = 0; i < 0x40; i++ );
	MAC_MCFG &= (~0x00008000);	/* Clear the reset */
	MAC_MCMD = 0;

	write_PHY (0, 0x1100); //Setup auto-negotiation, full duplex
	
	//tout = 0;
	//while(!(read_PHY(1) & 4) & (tout < 500000))	//check and wait until ethernet is up
	//	tout++;
	//	
	MAC_SA0 = (MYMAC_1 << 8) | MYMAC_2;
	MAC_SA1 = (MYMAC_3 << 8) | MYMAC_4;
	MAC_SA2 = (MYMAC_5 << 8) | MYMAC_6;
	
	return 1;	
}
